/*=============================================================================
  Copyright (c) 2001-2011 Joel de Guzman
  Copyright (c) 2001-2011 Hartmut Kaiser
  http://spirit.sourceforge.net/

  Distributed under the Boost Software License, Version 1.0. (See accompanying
  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
=============================================================================*/
#if !defined(SPIRIT_VERSION_NOVEMBER_13_2008_0834AM)
#define SPIRIT_VERSION_NOVEMBER_13_2008_0834AM

#include <boost/spirit/include/version.hpp>

#endif
